@@ -1,4 +1,4 @@
-# copyright 2004-2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2004-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This file is part of yams.
@@ -24,7 +24,7 @@ __docformat__ = "restructuredtext en"
 modname = 'yams'
 
 # release version
-numversion = (0, 38, 1)
+numversion = (0, 39, 0)
 version = '.'.join(str(num) for num in numversion)
 
 # license and copyright
@@ -60,5 +60,5 @@ scripts = ['bin/yams-check', 'bin/yams-view']
 
 install_requires = [
     'logilab-common >= 0.55.2',
-    'logilab-database >= 1.5.0',
+    'logilab-database >= 1.11',
     ]
