@@ -31,7 +31,7 @@ from logilab.common.interface import implements
 
 import yams
 from yams import (BASE_TYPES, MARKER, ValidationError, BadSchemaDefinition,
-                  KNOWN_METAATTRIBUTES)
+                  KNOWN_METAATTRIBUTES, convert_default_value)
 from yams.interfaces import (ISchema, IRelationSchema, IEntitySchema,
                              IVocabularyConstraint)
 from yams.constraints import BASE_CHECKERS, BASE_CONVERTERS, UniqueConstraint
@@ -51,6 +51,8 @@ def check_permission_definitions(schema):
         assert isinstance(groups, tuple), \
                ('permission for action %s of %s isn\'t a tuple as '
                 'expected' % (action, schema))
+    if schema.final:
+        schema.advertise_new_add_permission()
     for action in schema.ACTIONS:
         assert action in schema.permissions, \
                'missing expected permissions for action %s for %s' % (action, schema)
@@ -91,7 +93,6 @@ def _format_properties(props):
     res = [('%s=%s' % item) for item in props.items() if item[1]]
     return ','.join(res)
 
-
 class ERSchema(object):
     """Base class shared by entity and relation schema."""
 
@@ -109,6 +110,7 @@ class ERSchema(object):
         self.schema = schema
         self.type = erdef.name
         self.description = erdef.description or ''
+        self.package = erdef.package
 
     def __cmp__(self, other):
         return cmp(self.type, getattr(other, 'type', other))
@@ -207,6 +209,10 @@ class EntitySchema(PermissionMixIn, ERSchema):
         self.subjrels = rehash(self.subjrels)
         self.objrels = rehash(self.objrels)
 
+    def advertise_new_add_permission(self):
+        pass
+
+
     # schema building methods #################################################
 
     def add_subject_relation(self, rschema):
@@ -375,27 +381,7 @@ class EntitySchema(PermissionMixIn, ERSchema):
         if default is MARKER:
             default = None
         elif default is not None:
-            # rdef.object is the attribute type
-            if rdef.object == 'Boolean':
-                if not isinstance(default, bool):
-                    default = default == 'True' # XXX duh?
-            elif rdef.object in ('Int', 'BigInt'):
-                if not isinstance(default, (int, long)):
-                    default = int(default)
-            elif rdef.object == 'Float':
-                if not isinstance(default, float):
-                    default = float(default)
-            elif rdef.object == 'Decimal':
-                if not isinstance(default, Decimal):
-                    default = Decimal(default)
-            elif rdef.object in ('Date', 'Datetime', 'Time'):
-                key = '%s.%s' % (rdef.object, default.upper())
-                try:
-                    default = yams.KEYWORD_MAP[key]()
-                except KeyError:
-                    default = yams.DATE_FACTORY_MAP[rdef.object](default)
-            else:
-                default = unicode(default)
+            return convert_default_value(rdef, default)
         return default
 
     def has_unique_values(self, rtype):
@@ -731,7 +717,8 @@ class RelationSchema(ERSchema):
         if key in self.rdefs:
             msg = '(%s, %s) already defined for %s' % (subject, object, self)
             raise BadSchemaDefinition(msg)
-        self.rdefs[key] = rdef = RelationDefinitionSchema(subject, self, object)
+        self.rdefs[key] = rdef = RelationDefinitionSchema(subject, self, object,
+                                                          buildrdef.package)
         for prop, default in rdef.rproperties().iteritems():
             rdefval = getattr(buildrdef, prop, MARKER)
             if rdefval is MARKER:
@@ -836,17 +823,18 @@ class RelationDefinitionSchema(PermissionMixIn):
                          cls._FINAL_RPROPERTIES,
                          *cls.BASE_TYPE_PROPERTIES.itervalues()))
 
-    def __init__(self, subject, rtype, object, values=None):
+    def __init__(self, subject, rtype, object, package, values=None):
         if values is not None:
             self.update(values)
         self.subject = subject
         self.rtype = rtype
         self.object = object
+        self.package = package
 
     @property
     def ACTIONS(self):
         if self.rtype.final:
-            return ('read', 'update')
+            return ('read', 'add', 'update')
         else:
             return ('read', 'add', 'delete')
 
@@ -867,6 +855,26 @@ class RelationDefinitionSchema(PermissionMixIn):
     def as_triple(self):
         return (self.subject, self.rtype, self.object)
 
+    def advertise_new_add_permission(self):
+        """handle backward compatibility with pre-add permissions
+
+        * if the update permission was () [empty tuple], use the
+          default attribute permissions for `add`
+
+        * else copy the `update` rule for `add`
+        """
+        if not 'add' in self.permissions:
+            from yams.buildobjs import DEFAULT_ATTRPERMS
+            if self.permissions['update'] == ():
+                defaultaddperms = DEFAULT_ATTRPERMS['add']
+            else:
+                defaultaddperms = self.permissions['update']
+            self.permissions['add'] = defaultaddperms
+            warnings.warn('[yams 0.39] %s: new "add" permissions on attribute '
+                          'set to %s by default, but you must make it explicit' %
+                          (self, defaultaddperms), DeprecationWarning)
+
+
     @classmethod
     def rproperty_defs(cls, desttype):
         """return a dictionary mapping property name to its definition for each
@@ -896,6 +904,7 @@ class RelationDefinitionSchema(PermissionMixIn):
 
     def dump(self, subject, object):
         return RelationDefinitionSchema(subject, self.rtype, object,
+                                        self.package,
                                         self.__dict__)
 
     def role_cardinality(self, role):
